//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "stdafx.h"
#include "emule.h"
#include "packets.h"
#include "ServerSocket.h"
#include "emuleDlg.h"
#include "opcodes.h"
#include "SearchList.h"
#include <time.h>
#include "otherfunctions.h"
#include "Smtp.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#ifdef OLD_SOCKETS_ENABLED

CServerSocket::CServerSocket(CServerConnect* in_serverconnect)
{
	EMULE_TRY

	m_pServerConnect = in_serverconnect;
	m_eConnectionState = CS_NOTCONNECTED;
	m_pServer = NULL;
	m_strInfo="";
	m_iOldID = 0;
	m_dwMessageCount= 0;
	m_dwLastTransmission = 0;

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CServerSocket::~CServerSocket()
{
	delete m_pServer;
	m_pServer = NULL;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	OnConnect() handles the "connect" socket notification (dispatched from AsyncSocketExHelperWindow).
void CServerSocket::OnConnect(int nErrorCode)
{
	EMULE_TRY

	CAsyncSocketEx::OnConnect(nErrorCode);

	switch (nErrorCode)
	{
		case 0:	// No error
		{
			if (m_pServer->HasDynIP())
			{
				SOCKADDR_IN		sockAddr;

				memzero(&sockAddr, sizeof(sockAddr));

				int		iSockAddrLen = sizeof(sockAddr);

				GetPeerName(reinterpret_cast<SOCKADDR*>(&sockAddr),&iSockAddrLen);
				m_pServer->SetID(sockAddr.sin_addr.S_un.S_addr);
			//
			//	Update the Server with the actual IP that we connected to.
				CServer		*pServer = NULL;

				pServer = g_App.m_pServerList->GetServerByAddress(m_pServer->GetAddress(),m_pServer->GetPort());
				pServer->SetID(sockAddr.sin_addr.S_un.S_addr);
			}
		//	We're waiting for the server to log us in.
			SetConnectionState(CS_WAITFORLOGIN);
			m_dwMessageCount = 0;
			break;
		}
		case WSAEADDRNOTAVAIL:	// Various errors which basically mean "we couldn't connect to the server".
		case WSAECONNREFUSED:
		case WSAENETUNREACH:
		case WSAETIMEDOUT:
		case WSAEADDRINUSE:
			SetConnectionState(CS_SERVERDEAD);
			return;
		case WSAECONNABORTED:
			if (m_ProxyConnectFailed)
			{
				m_ProxyConnectFailed = false;
				SetConnectionState(CS_SERVERDEAD);
				return;
			}
		default:
		//	Some error occurred which we don't understand
			SetConnectionState(CS_FATALERROR);
			return;
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	OnReceive() handles the "receive" socket notification (dispatched from AsyncSocketExHelperWindow).
void CServerSocket::OnReceive(int nErrorCode)
{
//	If we're not connected or connecting, ignore this.
	if (m_eConnectionState != CS_CONNECTED && !this->m_pServerConnect->IsConnecting())
	{
		return;
	}
	CEMSocket::OnReceive(nErrorCode);
	m_dwLastTransmission = GetTickCount();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CServerSocket::ProcessPacket(char* pcPacketBuf, uint32 dwPacketSize, EnumOpcodes eOpcode)
{
	try
	{
		switch(eOpcode)
		{
			case OP_SERVERMESSAGE:
			{
			//	Fixed "OP_SERVERMESSAGE Format String Vulnerability" - from official 0.29b - bond006
				CString		strMessages;

				if (dwPacketSize >= 2)
				{
					UINT	uLen = *reinterpret_cast<uint16*>(pcPacketBuf);

					if (uLen > dwPacketSize-2)
						uLen = dwPacketSize-2;
					memcpy2(strMessages.GetBuffer(uLen), pcPacketBuf + sizeof(uint16), uLen);
					strMessages.ReleaseBuffer(uLen);
				}

			//	16.40 servers do not send separate OP_SERVERMESSAGE packets for each line;
			//	instead of this they are sending all text lines with one OP_SERVERMESSAGE packet.
				int			iPos = 0;
				CString		strMessage = strMessages.Tokenize("\r\n", iPos);

				while (!strMessage.IsEmpty())
				{
					m_dwMessageCount++;
					bool bOutputMessage = true;

					if (strnicmp(strMessage, "server version", 14) == 0)
					{
						CString strVer = strMessage.Mid(14);
						strVer.Trim();
						strVer = strVer.Left(64); // truncate string to avoid misuse by servers in showing ads
						CServer* pServer = g_App.m_pServerList->GetServerByAddress(m_pServer->GetAddress(),m_pServer->GetPort());
						if (pServer)
						{
							pServer->SetVersion(strVer);
							g_App.m_pMDlg->m_wndServer.m_ctlServerList.RefreshServer(*pServer);
						}
					}
					else if (strncmp(strMessage, "ERROR", 5) == 0)
					{
						CServer* pServer = g_App.m_pServerList->GetServerByAddress(m_pServer->GetAddress(),m_pServer->GetPort());
						AddLogLine(true, RGB_LOG_ERROR + _T("ERROR %s (%s:%u) - %s"),
							pServer ? pServer->GetListName() : GetResString(IDS_PW_SERVER),
							m_pServer->GetAddress(), m_pServer->GetPort(), strMessage.Mid(5).Trim(_T(" :")));
						bOutputMessage = false;
					}
					else if (strncmp(strMessage, "WARNING", 7) == 0){
						CServer* pServer = g_App.m_pServerList->GetServerByAddress(m_pServer->GetAddress(),m_pServer->GetPort());
						AddLogLine(true, RGB_LOG_WARNING + _T("WARNING %s (%s:%u) - %s"),
							pServer ? pServer->GetListName() : GetResString(IDS_PW_SERVER),
							m_pServer->GetAddress(), m_pServer->GetPort(), strMessage.Mid(7).Trim(_T(" :")));
						bOutputMessage = false;
					}

					int		iIdx1 = strMessage.Find("[emDynIP: "), iIdx2 = strMessage.Find(']');

					if ((iIdx1 != -1) && (iIdx1 < iIdx2))
					{
						CString strDynIP = strMessage.Mid(iIdx1 + 10, iIdx2 - (iIdx1 + 10));
						strDynIP.Trim(' ');
						if (strDynIP.GetLength() && strDynIP.GetLength() < 51)
						{
							CServer* pServer = g_App.m_pServerList->GetServerByAddress(m_pServer->GetAddress(),m_pServer->GetPort());
							if (pServer)
							{
								pServer->SetDynIP(strDynIP.GetBuffer());
								m_pServer->SetDynIP(strDynIP.GetBuffer());
								g_App.m_pMDlg->m_wndServer.m_ctlServerList.RefreshServer(*pServer);
							}
						}
					}

					if (bOutputMessage)
						g_App.m_pMDlg->AddServerMessageLine(strMessage);

					strMessage = strMessages.Tokenize("\r\n", iPos);
				}
				break;
			}
			case OP_IDCHANGE:
			{
				if (dwPacketSize < sizeof(LoginAnswer_Struct))
					throw CString(_T("corrupt or invalid login answer from server received"));

				LoginAnswer_Struct* p_sLoginAnswer = (LoginAnswer_Struct*) pcPacketBuf;
				byte state;

				// get & save TCP flags for current server
				ASSERT( m_pServer );
				if (m_pServer)
				{
					if (dwPacketSize >= sizeof(LoginAnswer_Struct) + 4)
						m_pServer->SetTCPFlags(*((uint32*)(pcPacketBuf + sizeof(LoginAnswer_Struct))));
					else
						m_pServer->SetTCPFlags(0);

					// copy TCP flags into the server in the server list
					CServer* pServer = g_App.m_pServerList->GetServerByAddress(m_pServer->GetAddress(), m_pServer->GetPort());
					if (pServer)
						pServer->SetTCPFlags(m_pServer->GetTCPFlags());

					if (dwPacketSize >= sizeof(LoginAnswer_Struct) + 8)
					{
						uint16	uPort = static_cast<uint16>(*((uint32*)(pcPacketBuf + sizeof(LoginAnswer_Struct) + 4)));
						if (m_pServer->GetPort() != uPort)
							m_pServer->SetAuxPort(m_pServer->GetPort());
						else if (m_pServer->GetAuxPort() == uPort)
							m_pServer->SetAuxPort(0);
						m_pServer->SetPort(uPort);

					//	Refresh the servers infos in the servers list
						CServer	   *pServer = g_App.m_pServerList->GetServerByAddress(m_pServer->GetAddress(), m_pServer->GetPort());
						if (pServer != NULL)
						{
							pServer->SetAuxPort(m_pServer->GetAuxPort());
							pServer->SetPort(m_pServer->GetPort());
						}
					}
				}

				if (p_sLoginAnswer->m_dwClientID == 0)
				{
					state = g_App.m_pPrefs->GetSmartIdState();
					if ( state > 0 )
					{
						state++;
						if( state > 3 )
							g_App.m_pPrefs->SetSmartIdState(0);
						else
							g_App.m_pPrefs->SetSmartIdState(state);
					}
					break;
				}

				if( g_App.m_pPrefs->GetSmartIdCheck() )
				{
					if (p_sLoginAnswer->m_dwClientID >= 0x1000000 )
						g_App.m_pPrefs->SetSmartIdState(1);
					else
					{
						state = g_App.m_pPrefs->GetSmartIdState();
						if ( state > 0 )
						{
							//SetConnectionState(CS_ERROR);
							state++;
							if( state > 3 )
							{
								g_App.m_pPrefs->SetSmartIdState(0);
							//	Sending message when connected with lowid
								CString strMessageText = GetResString(IDS_GOTLOWID);

								g_App.m_pSMTPConnection->SendMail(strMessageText, g_App.m_pPrefs->GetNotifierPopOnServerError(), g_App.m_pPrefs->IsSMTPWarningEnabled());
								g_App.m_pMDlg->ShowNotifier(strMessageText, TBN_SERVER, false, g_App.m_pPrefs->GetNotifierPopOnServerError());
							}
							else
							{
								SetConnectionState(CS_ERROR);
								g_App.m_pMDlg->AddLogLine(false, GetResString(IDS_GOTLOWID));		//Cax2 - smartId bugfix
								g_App.m_pPrefs->SetSmartIdState(state);
							}
							break;
						}
					}
				}

				if (m_eConnectionState != CS_CONNECTED)
				{
					SetConnectionState(CS_CONNECTED);
					g_App.OnlineSig();       // Added By Bouc7
				}
				m_pServerConnect->SetClientID(p_sLoginAnswer->m_dwClientID);

				if (p_sLoginAnswer->m_dwClientID != m_iOldID)
				{
					// Purity - Sending message when ID change
					CString strMessageText;
					strMessageText.Format(GetResString(IDS_NEWCLIENTID),p_sLoginAnswer->m_dwClientID);
					g_App.m_pSMTPConnection->SendMail(strMessageText, g_App.m_pPrefs->GetNotifierPopOnServerError(), g_App.m_pPrefs->IsSMTPWarningEnabled());
				}
				m_iOldID = p_sLoginAnswer->m_dwClientID;
				AddLogLine(false, IDS_NEWCLIENTID, p_sLoginAnswer->m_dwClientID);

				g_App.m_pDownloadQueue->ResumeFiles();
				break;
			}
			case OP_SEARCHRESULT:
			{
				bool bIsMoreResultsAvailable;
				uint16 uResultsCount = g_App.m_pSearchList->ProcessSearchAnswer(pcPacketBuf, dwPacketSize, &bIsMoreResultsAvailable);
				g_App.m_pMDlg->m_dlgSearch.LocalSearchEnd(uResultsCount, bIsMoreResultsAvailable);
				break;
			}
			case OP_FOUNDSOURCES:
			{
				CMemFile		packetStream(reinterpret_cast<BYTE*>(pcPacketBuf),dwPacketSize);
				uchar			fileHash[16];

				packetStream.Read(fileHash,16);

				CPartFile		*pPartFile = g_App.m_pDownloadQueue->GetFileByID(fileHash);

				if (pPartFile)
				{
					pPartFile->AddSources(packetStream,m_pServer->GetIP(), m_pServer->GetPort());
				}
				break;
			}
			case OP_SERVERSTATUS:
			{
				// FIXME some status packets have a different size -> why? structure?
				if (dwPacketSize < 8)
					break;//throw "Invalid status packet";
				uint32 uCurrentUsersNumber;
				uint32 uCurrentFilesNumber;
				memcpy2(&uCurrentUsersNumber,pcPacketBuf,4);
				memcpy2(&uCurrentFilesNumber,pcPacketBuf+4,4);
				CServer* pServer = g_App.m_pServerList->GetServerByAddress( m_pServer->GetAddress(), m_pServer->GetPort() );
				if (pServer)
				{
					pServer->SetUserCount(uCurrentUsersNumber);
					pServer->SetFileCount(uCurrentFilesNumber);
					g_App.m_pMDlg->ShowUserCount(uCurrentUsersNumber, uCurrentFilesNumber);
					g_App.m_pMDlg->m_wndServer.m_ctlServerList.RefreshServer(*pServer);
				}
				break;
			}
			case OP_SERVERIDENT:
			{
				// OP_SERVERIDENT - this is sent by the server only if we send a OP_GETSERVERLIST
				if (dwPacketSize<16+4+2+4)
				{
					AddDebugLogLine(RGB_LOG_WARNING + _T("Unknown server info received!"));
				//	Throw "Invalid server info received";
					break;
				}

				byte aucHash[16];
				uint32 nServerIP;
				uint16 nServerPort;
				uint32 nTags;
				CSafeMemFile data((BYTE*)pcPacketBuf, dwPacketSize);

				data.Read(aucHash, 16);			// <hash 16>
				data.Read(&nServerIP, 4);		// <serverip 4>
				data.Read(&nServerPort, 2);		// <serverport 2>
				data.Read(&nTags, 4);			// <tagcount 4>

				CString strName;
				CString strDescription;
				for (UINT i = 0; i < nTags; i++)
				{
					CTag tag;

					tag.FillFromStream(data);
					if (tag.GetTagID() == ST_SERVERNAME)
					{
						if (tag.IsStr())
						{
							tag.GetStringValue(&strName);
							strName.Remove(_T('\b'));
							strName.Remove(_T('\r'));
							strName.Remove(_T('\t'));
							strName.Trim();
						}
					}
					else if (tag.GetTagID() == ST_DESCRIPTION)
					{
						if (tag.IsStr())
						{
							tag.GetStringValue(&strDescription);
							strDescription.Remove(_T('\b'));
							strDescription.Remove(_T('\r'));
							strDescription.Remove(_T('\t'));
							strDescription.Trim();
						}
					}
				}

				ASSERT( m_pServer );
				if (m_pServer)
				{
					CServer* pServer = g_App.m_pServerList->GetServerByAddress(m_pServer->GetAddress(),m_pServer->GetPort());
					ASSERT( pServer );
					if (pServer)
					{
						if(!pServer->IsStaticMember())
							pServer->SetListName(strName);
						pServer->SetDescription(strDescription);
						if (((uint32*)aucHash)[0] == 0x2A2A2A2A)
						{
							const CString& rstrVersion = pServer->GetVersion();
							if (!rstrVersion.IsEmpty())
								pServer->SetVersion(_T("eFarm ") + rstrVersion);
							else
								pServer->SetVersion(_T("eFarm"));
						}
						g_App.m_pMDlg->ShowConnectionState(true,pServer->GetListName());
						g_App.m_pMDlg->m_wndServer.m_ctlServerList.RefreshServer(*pServer);
					}
				}
				break;
			}
		//	Add server's Server List to own Server List
			case OP_SERVERLIST:
			{
				try
				{
					CSafeMemFile		packetStream((BYTE*)pcPacketBuf,dwPacketSize);

					byte		byteNumServers;

					packetStream.Read(&byteNumServers, 1);

				//	Verify the packet size
					if ((byteNumServers*6u + 1u) > dwPacketSize)	 // (<ip 4><port 2>)*server count, <server count 1>
					{
						byteNumServers = 0;
					}

					int		iAddedServers = 0;
					uint32	dwIP;
					uint16	uPort;

					while (byteNumServers != 0)
					{
						packetStream.Read(&dwIP, 4);
						packetStream.Read(&uPort, 2);

						in_addr		host;

						host.S_un.S_addr = dwIP;

						CString		strIP(inet_ntoa(host));
						CServer		*pNewServer = new CServer(uPort, strIP);

						pNewServer->SetListName(pNewServer->GetFullIP());
						if (!g_App.m_pMDlg->m_wndServer.m_ctlServerList.AddServer(pNewServer, true))
						{
							delete pNewServer;
						}
						else
						{
							iAddedServers++;
						}
						byteNumServers--;
					}
					if (iAddedServers)
					{
						AddLogLine(false, IDS_NEWSERVERS, iAddedServers);
					}
				}
			//	CFileException change to CException cause "new" also can be a reason for exception
				catch (CException *error)
				{
					OUTPUT_DEBUG_TRACE();
					AddDebugLogLine(RGB_LOG_WARNING + _T("Invalid serverlist packet received"));
					error->Delete();
				}
				break;
			}
			case OP_CALLBACKREQUESTED:
			{
			//	If the packet is the right size...
				if (dwPacketSize == 6)
				{
					uint32		dwIP;
					uint16		nPort;

					memcpy2(&dwIP,pcPacketBuf,4);
					memcpy2(&nPort,pcPacketBuf+4,2);

					CUpDownClient		*pClient = g_App.m_pClientList->FindClientByIP(dwIP,nPort);

				//	If we already know of the client, make sure we're connected to him
					if (pClient != NULL)
					{
						pClient->TryToConnect();
					}
				//	If this client is new to us...
					else
					{
						try
						{
							pClient = new CUpDownClient(nPort,dwIP,0,0,0);
						//	Add the new client to our client list. If successful...
							if (g_App.m_pClientList->AddClient(pClient))
							{
								pClient->SetUserName(GetResString(IDS_SERVER_SOURCE));
								pClient->TryToConnect();
							}
							else
							{
								safe_delete(pClient);
							}
						}
						catch(CMemoryException* error)
						{
							OUTPUT_DEBUG_TRACE();
							error->Delete();
						}
					}
				}
				break;
			}
			case OP_REJECT:
			{
			//  This could happen if we send a command with the wrong protocol
			//	(e.g. sending a compressed packet to a server which does not support that protocol)
				AddDebugLogLine(RGB_LOG_WARNING + _T("Server rejected last command"));
				break;
			}
			default:
				;
		}
		return true;
	}
	catch(CFileException* error)
	{
		OUTPUT_DEBUG_TRACE();
		error->m_strFileName = "server packet";
		AddDebugLogLine(RGB_LOG_WARNING + _T("Unhandled error while processing packet from server (%s)"), GetErrorMessage(error));
		error->Delete();
		if (eOpcode == OP_SEARCHRESULT || eOpcode == OP_FOUNDSOURCES)
			return true;
	}
	catch(CMemoryException* error)
	{
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(RGB_LOG_WARNING + _T("Unhandled error while processing packet from server (CMemoryException)"));
		error->Delete();
		if (eOpcode == OP_SEARCHRESULT || eOpcode == OP_FOUNDSOURCES)
			return true;
	}
	catch(CString error)
	{
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(RGB_LOG_WARNING + _T("Unhandled error while processing packet from server (%s)"), error.GetBuffer());
	}
	catch(...)
	{
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(RGB_LOG_WARNING + _T("Unhandled error while processing packet from server (Unknown exception)"));
	}

	SetConnectionState(CS_DISCONNECTED);
	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CServerSocket::ConnectToServer(CServer &server)
{
	EMULE_TRY

	if (&server == NULL)
		return;

//	Check 'm_pServer': if object already exist, then replace it
	safe_delete(m_pServer);
//	Make a local copy of 'server'.
	m_pServer = new CServer(server);

	AddLogLine(false, IDS_CONNECTINGTO, m_pServer->GetListName(), m_pServer->GetFullIP(), m_pServer->GetPort());

//	If we're using proxies but "server connect without proxy" is set in Preferences...
	if (g_App.m_pPrefs->IsProxyASCWOP())
	{
		if (g_App.m_pPrefs->GetProxy().m_bUseProxy == true)
		{
			g_App.m_pPrefs->SetProxyASCWOP(true);
			g_App.m_pPrefs->SetUseProxy(false);
			AddLogLine(false, _T("%s: %s"), GetResString(IDS_SERVERCON_WO_PROXY), GetResString(IDS_DISABLED).MakeLower());
		}
		else
			g_App.m_pPrefs->SetProxyASCWOP(false);
	}
	SetConnectionState(CS_CONNECTING);

//	Ensure that we're listening for the server's listening port connection attempt.
	g_App.m_pListenSocket->RestartListening();

//	Try to connect to 'server'. If there's an error...
	uint16	uConnPort;

	if (g_App.m_pPrefs->IsServerAuxPortUsed() && server.GetAuxPort() != 0)
	{
		uConnPort = server.GetAuxPort();
	}
	else
	{
		uConnPort = server.GetPort();
	}

	if (!this->Connect(server.GetAddress(),uConnPort))
	{
		int		iError = this->GetLastError();

		if (iError != WSAEWOULDBLOCK)
		{
			AddLogLine(false, IDS_ERR_CONNECTIONERROR, m_pServer->GetListName(),m_pServer->GetFullIP(), uConnPort, iError);
		//	Set the connection state and notify our ServerConnect that the connection has failed.
			SetConnectionState(CS_FATALERROR);
			return;
		}
	}
//	If we connected successfully...
	else
	{
	//	Remember the server name/IP string.
		m_strInfo = server.GetListName();
	//	Set the connection state and notify our ServerConnect that the connection has been established.
		SetConnectionState(CS_CONNECTED);
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CServerSocket::OnError(int nErrorCode)
{
	EMULE_TRY

	SetConnectionState(CS_DISCONNECTED);
	AddDebugLogLine( RGB_LOG_ERROR + _T("Error in serversocket: %s (%s:%i): %u"),
					 m_pServer->GetListName(), m_pServer->GetFullIP(), m_pServer->GetPort(), nErrorCode );

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CServerSocket::PacketReceived(Packet* pPacket)
{
	EMULE_TRY

//	Calculate overhead at one place before decompression
	g_App.m_pDownloadQueue->AddDownDataOverheadServer(pPacket->m_dwSize);

	if (pPacket->m_eProtocol == OP_PACKEDPROT)
	{
		if (!pPacket->UnpackPacket(250000))
		{
			AddDebugLogLine(RGB_LOG_WARNING + _T("Failed to decompress server TCP packet: protocol=0x%02x  opcode=0x%02x  size=%u"),
				pPacket ? pPacket->m_eProtocol : 0,
				pPacket ? pPacket->m_eOpcode : 0,
				pPacket ? pPacket->m_dwSize : 0);
			return;
		}
		pPacket->m_eProtocol = OP_EDONKEYPROT;
	}

	if (pPacket->m_eProtocol == OP_EDONKEYPROT)
		ProcessPacket(pPacket->m_pcBuffer,pPacket->m_dwSize,pPacket->m_eOpcode);
	else
		AddDebugLogLine(RGB_LOG_WARNING + _T("Received server TCP packet with unknown protocol: protocol=0x%02x  opcode=0x%02x  size=%u"),
			pPacket ? pPacket->m_eProtocol : 0,
			pPacket ? pPacket->m_eOpcode : 0,
			pPacket ? pPacket->m_dwSize : 0);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CServerSocket::OnClose(int nErrorCode)
{
	CEMSocket::OnClose(0);
	if (m_eConnectionState == CS_WAITFORLOGIN)
		SetConnectionState(CS_SERVERFULL);
	else if (m_eConnectionState == CS_CONNECTED)
		SetConnectionState(CS_DISCONNECTED);
	else
		SetConnectionState(CS_NOTCONNECTED);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	SetConnectionState() sets the connection state of this server socket and notifies
//	its associated ServerConnect of the change in state.
void CServerSocket::SetConnectionState(EnumServerConnectionStates eNewState)
{
	EMULE_TRY

	m_eConnectionState = eNewState;

	if (eNewState < 1)
	{
		m_pServerConnect->ConnectionFailed(this);
	}
	else if (eNewState == CS_CONNECTED || eNewState == CS_WAITFORLOGIN)
	{
		if (m_pServerConnect)
			m_pServerConnect->ConnectionEstablished(this);
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif OLD_SOCKETS_ENABLED
